@@ -11,7 +11,8 @@ from typing import List, Union
 import magic
 import requests
 from ucode.helpers.clog import CLog
-from ucode.helpers.misc import make_slug
+from ucode.helpers.config import Config
+from ucode.helpers.misc import make_slug, dos2unix
 from ucode.models.problem import Problem, TestCase
 from ucode.models.question import Question, QuestionType
 from ucode.services.dsa.problem_service import ProblemService
@@ -20,14 +21,66 @@ _logger = logging.getLogger(__name__)
 
 
 class UCode:
-    def __init__(self, base_url, token):
+    def __init__(self, env="prod"):
         self.s = requests.session()
-        self.api_base_url = base_url
-        self.token = token
+        self.env = env
+        self._headers = self.token = None
+        if env == "prod":
+            self.api_base_url = "https://api.ucode.vn/api"
+        else:
+            self.api_base_url = "https://dev-api.ucode.vn/api"
+        self.load_config()
+
+    def load_config(self):
+        config = Config.get_instance().load()
+        credentials = config.get("credentials")
+        if not credentials or not credentials.get(self.env):
+            return
+        self.token = credentials[self.env].get("access-token")
         self._headers = {
             'access-token': self.token
         }
 
+    def login(self, email, password):
+        url = self.get_api_url(f"/users/sign-in")
+        data = {
+            "email": email,
+            "password": password
+        }
+        response = self.s.post(url, json=data)
+        res_json = json.loads(response.text)
+        if response.status_code != 200:
+            CLog.error(f"Login Failed: {res_json['message']}")
+            return None
+
+        self.token = res_json["data"]
+
+        config = Config.get_instance().load()
+        if "credentials" not in config:
+            config["credentials"] = {}
+        config["credentials"][self.env] = {
+            "env": self.env,
+            "access-token": self.token
+        }
+
+        Config.get_instance().save(config)
+        CLog.info(f"User `{email}` logged in successfully")
+
+    def get_logged_in_user(self):
+        self.load_config()
+        if not self.token:
+            CLog.error("Please login with ucode email and password first!")
+            return None
+        url = self.get_api_url("/users/info")
+        response = self.s.get(url, headers=self._headers)
+        res_json = json.loads(response.text)
+
+        if response.status_code == 401:
+            CLog.error(f"Access token expired, please re-login with valid username and password.")
+            return None
+
+        print(f"Logged in `{self.env}` env with user `{res_json['data']['email']}`!")
+
     def upload_file(self, file_path, upload_folder):
         url = self.get_api_url(f"/general/upload-file?folder={upload_folder}")
         # url = "https://httpbin.org/post"
@@ -234,23 +287,23 @@ class UCode:
     def create_problems(self, lesson_id, problems: List[Problem], score=10, xp=100, lang="vi"):
         question_ids = []
         for problem in problems:
-            q_id = self.create_problem(self, lesson_id=lesson_id,
+            q_id = self.create_problem(lesson_id=lesson_id,
                                        problem=problem,
                                        score=score, xp=xp, lang=lang)
             question_ids.append(q_id)
 
         return question_ids
 
-    def upload_testcases(self, problem_id, testcases: List[TestCase], samples=None, score=10):
+    def upload_testcases(self, problem_id, testcases: List[TestCase], score=10):
         url = self.get_api_url(f"/question/{problem_id}/upload-testcases")
         testcase_data = []
         for i, testcase in enumerate(testcases):
             t = {
                 "name": testcase.name,
                 "explanation": testcase.explanation,
-                "input": testcase.input,
-                "output": testcase.output,
-                "score": score,
+                "input": dos2unix(testcase.input),
+                "output": dos2unix(testcase.output),
+                "score": 0 if testcase.sample else score,
                 "is_sample": testcase.sample
             }
             testcase_data.append(t)
@@ -273,9 +326,9 @@ class UCode:
         data = {
             "name": testcase.name,
             "explanation": testcase.explanation,
-            "input": testcase.input,
-            "output": testcase.output,
-            "score": score,
+            "input": dos2unix(testcase.input),
+            "output": dos2unix(testcase.output),
+            "score": 0 if is_sample else score,
             "is_sample": bool(is_sample)
         }
 
@@ -389,7 +442,7 @@ class UCode:
 
 
 def create_chapters_mc1():
-    ucode = UCode("https://dev-api.ucode.vn/api", "72821b59462c5fdb552a049c1caed85c")
+    ucode = UCode("dev")
 
     course_id = 7
     for w in range(1, 13):
@@ -398,10 +451,14 @@ def create_chapters_mc1():
 
 
 def create_public_problem():
-    problems = ProblemService.read_all_problems_v1("/home/thuc/projects/ucode/weekly-algorithm-problems/week13",
-                                                   translations=["vi"], load_testcase=True)
+    # problems = ProblemService.read_all_problems_v1("/home/thuc/projects/ucode/weekly-algorithm-problems/week13",
+    #                                                translations=["vi"], load_testcase=True)
+    # problem_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\quan_ba_dinh_2020_2021_thcs_round2\\p03_capdoixung"
+    problem_folder = "D:\\projects\\ucode\\dsa-problems\\competitions\\hsg_thcs\\Hieu\\HCM2018_new\\B3"
+    problem = ProblemService.load(problem_folder, load_testcase=True)
+    problems = [(problem_folder, problem)]
 
-    ucode = UCode("https://dev-api.ucode.vn/api", "743c1fce2615a4cb569dd57f86b73898")
+    ucode = UCode("prod")
 
     print(len(problems))
     for i, (problem_folder, problem) in enumerate(problems):
@@ -409,42 +466,57 @@ def create_public_problem():
         # print(problem.tags)
         # print(problem.translations['vi'].statement)
         print("Testcases: ", len(problem.testcases))
-        res = ucode.create_problem(lesson_id=None, problem=problem, lang="en", xp=-1, status="draft", is_server_judge=True)
-        print("res:", res)
+        # res = ucode.create_problem(lesson_id=None, problem=problem, lang="en", xp=-1, status="draft", is_server_judge=True)
+        # res = ucode.create_problem(lesson_id=None, problem=problem, xp=-1)
+        # print("res:", res)
         # break
 
 
-def upload_all_problems_in_folder():
-    # lession_id = 4046
-    lession_id = 0 # public problem
-    base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\BaDinh_2021_9"
-    problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
-    print("Total problems:", len(problems))
-
-    ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
-    # ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
-
-    for i, (problem_folder, problem) in enumerate(problems):
-        problem = ProblemService.load(problem_folder, load_testcase=True)
-        print(i + 1, problem.name)
-        # ucode.create_problem(lesson_id=lession_id, problem=problem, xp=200)
-        print("testcases:", len(problem.testcases), len(problem.testcases_sample))
-        ucode_id = ucode.create_problem(lesson_id=lession_id, problem=problem, xp=-1)
-        print("uCode problem created: ", ucode_id)
+# def upload_all_problems_in_folder():
+#     lession_id = 4046
+#     # lession_id = 0 # public problem
+#     base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\BaDinh_2021_9"
+#     problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
+#     print("Total problems:", len(problems))
+#
+#     # ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
+#     ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
+#
+#     for i, (problem_folder, problem) in enumerate(problems):
+#         problem = ProblemService.load(problem_folder, load_testcase=True)
+#         print(i + 1, problem.name)
+#         # ucode.create_problem(lesson_id=lession_id, problem=problem, xp=200)
+#         print("testcases:", len(problem.testcases), len(problem.testcases_sample))
+#         ucode_id = ucode.create_problem(lesson_id=lession_id, problem=problem, xp=-1)
+#         print("uCode problem created: ", ucode_id)
 
 
 def create_problems_to_lesson_ucode_live():
-    base_folder = "D:\\projects\\ucode\\repl_backup\\ThucNguyen1\\py1-2-lesson-1-review"
-    problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
+    base_folder = "D:\\projects\\ucode\\repl_backup\\ThucNguyen1\\pya1-1-lesson-8-9-10-for"
 
-    lesson_id = 4052
-    ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
+    # base_folder = "D:\\projects\\ucode\\repl_backup\\itteacher-k1\\quiz5.1_list"
+    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\quan_ba_dinh_2020_2021_thcs_round2"
+    # base_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\competitions\\hsg-hn-2020_new"
+
+    # D:\projects\ucode\weekly-algorithm-problems\week13
+    base_folder = "D:\\projects\\ucode\\repl_backup\\itteacher-k1\\quiz8_dict"
+    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs"
+    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\vinhphuc_2020_2021_thcs"
+    # base_folder = "D:\\projects\\ucode\\repl_backup\\intro2"
+    # base_folder = "D:\\projects\\ucode\\dsa-problems\\competitions\\hsg_thcs\Thuc\HaNoi2019_new"
+    base_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\week13"
+    problems = ProblemService.read_all_problems_v1(base_folder, nested_folder=False, translations=['vi'])
+    # problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
+
+    lesson_id = 5477
+    ucode = UCode("prod")
     #
     # lesson_id = 4622
     # ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
 
     for i, (problem_folder, problem) in enumerate(problems):
-        problem = ProblemService.load(problem_folder, load_testcase=True)
+        # problem = ProblemService.load(problem_folder, load_testcase=True)
+        problem = ProblemService.load_v1(problem_folder, load_testcase=True, translations=['vi'])
         print(i + 1, problem.name)
         prob_id = ucode.create_problem(lesson_id=lesson_id, problem=problem, xp=-1)
         print(f"Problem uploaded with id #{prob_id}")
@@ -452,29 +524,45 @@ def create_problems_to_lesson_ucode_live():
     print("Total problems:", len(problems))
 
 
-if __name__ == "__main__":
-    # create_problems_to_lesson_ucode_live()
-    # upload_all_problems_in_folder()
-
-    # lession_id = 4051
-    lession_id = 0
-    #
-    # # ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
-    ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
-
-    problem_folder = "../../problems/a_plus_b"
+def reupload_testcases():
+    ucode = UCode("prod")
+    problem_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs\\p4_itable"
     problem: Problem = ProblemService.load(problem_folder, load_testcase=True)
 
-    # problem = ProblemService.load("D:\\projects\\ucode\\dsa-thematic-problems\\competitive_programing_topics\\"
-    #                               "maths\\fundamentals\\gcd_lcm\\fnum",
-    #                               load_testcase=True)
-    ucode.create_problem(lesson_id=lession_id, problem=problem, xp=-1)
-
-    print(problem.name)
+    question_id = 45486
+    ucode.upload_testcases(question_id, ProblemService.join_testcases(problem))
+    #
     print(len(problem.testcases))
     print(len(problem.testcases_sample))
-    print(problem.tags)
-    print(problem.resource_files)
+
+
+def test_login():
+    ucode = UCode(env="prod")
+    # ucode.login("gthuc.nguyen@gmail.com", "ngoaho85")
+    ucode.get_logged_in_user()
+
+
+if __name__ == "__main__":
+    # test_login()
+    # reupload_testcases()
+    create_problems_to_lesson_ucode_live()
+    # create_public_problem()
+    # upload_all_problems_in_folder()
+
+    # lession_id = 4068
+    # # lession_id = 0
+    # # #
+    # # # # ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
+    # ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
+    # # #
+    # problem_folder = "D:\\projects\\ucode\\dsa-problems\\unsorted\\thuc\\beautiful_pairs"
+    # problem: Problem = ProblemService.load(problem_folder, load_testcase=True)
+    #
+    # print(problem.name)
+    # print(len(problem.testcases))
+    # print(len(problem.testcases_sample))
+    # print(problem.tags)
+    # print(problem.resource_files)
     #
     # ucode_id = ucode.create_problem(lesson_id=0, problem=problem, xp=-1)
     # print("uCode problem created: ", ucode_id)
@@ -486,7 +574,6 @@ if __name__ == "__main__":
     # img_file = "C:/Users/thucnguyen/Downloads/z2154907105699_598b6d3a26a9aa817fc97c2c8079d261.jpg"
     # upload_url = ucode.upload_file(img_file, "thuc_test")
     # print(upload_url)
-    # create_public_problem()
     # ucode_courses = {
     #     2: 42,
     #     3: 43,
